Attribute VB_Name = "Neighbors"
'Count lexical neighbors.  This is not yet done because it doesn't have insertion and deletion.

    Option Explicit
    
    Dim mNumberOfWordsInDictionary As Long
    Dim mWords() As String
    Dim mWordsBySegments() As String
    Dim mWordLengths() As Long


Public Function CountNeighbors(InputWord As String)

    Dim Segments(gLengthOfLongestWord) As String
    Dim MySegment As String, MyWord As String, MyLength As Long
    Dim SuccessFlag As Boolean   'Signals neighbor by any of the three definitions (substitute, delete, insert)
    Dim Buffer As Long
    Dim NumberOfDifferences As Long
    Dim WordIndex As Long, SegmentIndex As Long
    
    Static DictionaryIsRead
    
    'Read the dictionary only the first time.
        If DictionaryIsRead = False Then
            Call ReadDictionary
            Let DictionaryIsRead = True
        End If
    
    'Initialize the report results string.
        Let BLICK.mReportResultsString = ""
    
    'Parse the word into segments.
        Let MyWord = InputWord
        Do
            Let MySegment = s.CustomChomp(MyWord, " ")
            Let MyWord = s.CustomResidue(MyWord, " ")
            'Quit if done.
                If MySegment = "" Then Exit Do
            'Vet the segment as ok.
                If BLICK.FindSegmentIndex(MySegment) = -1 Then
                    MsgBox "Caution:  the word " + MyWord + " has the illegal segment " + MySegment + "."
                End If
            'Augment the length of this word
                Let MyLength = MyLength + 1
            'Install the segment.
                Let Segments(MyLength) = MySegment
        Loop
        
    'Go through the dictionary looking for single-edit-distance neighbors.
        For WordIndex = 1 To mNumberOfWordsInDictionary
            'Consider only words of the same length, or differing by one.
                Let SuccessFlag = False
                Select Case mWordLengths(WordIndex)
                    Case MyLength + 1
                        'Check for neighbor-by-insertion.
                            If AddOneToAToGetB(InputWord, mWords(WordIndex)) Then Let SuccessFlag = True
                    Case MyLength
                        'Count the differences, looking for cases where there is precisely one difference.
                            Let NumberOfDifferences = 0
                            For SegmentIndex = 1 To MyLength
                                If Segments(SegmentIndex) <> mWordsBySegments(WordIndex, SegmentIndex) Then
                                    Let NumberOfDifferences = NumberOfDifferences + 1
                                    If NumberOfDifferences > 1 Then GoTo ExitPoint
                                End If
                            Next SegmentIndex
                            If NumberOfDifferences = 1 Then Let SuccessFlag = True
                    Case MyLength - 1
                        'Check for neighbor-by-deletion.
                            If AddOneToAToGetB(mWords(WordIndex), InputWord) Then Let SuccessFlag = True
                End Select                  'Select appropriate length relationships.
            'Remember a neighbor if you found one.
                If SuccessFlag Then
                    Let Buffer = Buffer + 1
                    If BLICK.mReportResultsString <> "" Then
                        Let BLICK.mReportResultsString = BLICK.mReportResultsString + ", "
                    End If
                    Let BLICK.mReportResultsString = BLICK.mReportResultsString + mWords(WordIndex)
                End If

ExitPoint:                                  'No need to keep considering this word; same length and two differences.
        Next WordIndex                      'Go through all words in the dictionary.
        
        Let CountNeighbors = Buffer
        
End Function

Function AddOneToAToGetB(ShorterWord As String, LongerWord As String) As Boolean

    'Store the parameters without altering them.
        Dim ShortBuffer As String, LongBuffer As String
    'For comparing individual segments.
        Dim SegmentOfShorterWord As String, SegmentOfLongerWord As String
    
    'Load the buffers:
        Let ShortBuffer = ShorterWord
        Let LongBuffer = LongerWord
    
    'Find the first difference:
        Do
            Let SegmentOfShorterWord = s.CustomChomp(ShortBuffer, " ")
            Let SegmentOfLongerWord = s.CustomChomp(LongBuffer, " ")
            If SegmentOfShorterWord <> SegmentOfLongerWord Then GoTo SecondStage
            Let LongBuffer = s.CustomResidue(LongBuffer, " ")
            Let ShortBuffer = s.CustomResidue(ShortBuffer, " ")
            If Trim(LongBuffer) = "" And Trim(ShortBuffer) = "" Then
                Let AddOneToAToGetB = False
                Exit Function
            End If
        Loop
        
SecondStage:
    'We assume that the mismatch is due to the shorter word missing a segment in the longer word.
    '   So, we take the residue of only the longer word, then assess identity.
        Let LongBuffer = s.CustomResidue(LongBuffer, " ")
        If Trim(LongBuffer) = Trim(ShortBuffer) Then
            Let AddOneToAToGetB = True
        Else
            Let AddOneToAToGetB = False
        End If

End Function



Sub ReadDictionary()

    Dim MyLine As String, MyWord As String, MySegment As String
    Dim DictFile As Long
    Dim WordCounter As Long
    
    Call FindLongestWord
    
    'Redimension the dictionary arrays.
        ReDim mWords(mNumberOfWordsInDictionary)
        ReDim mWordLengths(mNumberOfWordsInDictionary)
        ReDim mWordsBySegments(mNumberOfWordsInDictionary, gLengthOfLongestWord)
    
        Let DictFile = FreeFile
    
        Open App.Path + "/FilesNeededForOperation/Dictionary.txt" For Input As #DictFile
        Do While Not EOF(DictFile)
            Line Input #DictFile, MyLine
            Let MyLine = Trim(MyLine)
            Let WordCounter = WordCounter + 1
            Let MyLine = Trim(MyLine)
            'Grab and store the next word from this line, which also includes comments.
                Let MyWord = Trim(s.Chomp(MyLine))
                Let mWords(WordCounter) = MyWord
            'Break it up into segments.
                Do
                    Let MySegment = s.CustomChomp(MyWord, " ")
                    Let MyWord = s.CustomResidue(MyWord, " ")
                    'Quit if done.
                        If MySegment = "" Then Exit Do
                    'Vet the segment as ok.
                        If BLICK.FindSegmentIndex(MySegment) = -1 Then
                        '    MsgBox "Caution:  the dictionary word " + mWords(WordCounter) + ", the " + Trim(Str(WordCounter)) + "st word in the dictionary, has the bad segment " + MySegment + "."
                        End If
                    'Augment the length of this word
                        Let mWordLengths(WordCounter) = mWordLengths(WordCounter) + 1
                    'Install the segment.
                        Let mWordsBySegments(WordCounter, mWordLengths(WordCounter)) = MySegment
                Loop
        Loop
    Close #DictFile
    
    Exit Sub
    
    'Debug.
        Dim WordIndex As Long, SegmentIndex As Long
        Open App.Path + "/DebugDictionary.txt" For Output As #2
        For WordIndex = 1 To mNumberOfWordsInDictionary
            Print #2, mWords(WordIndex); Chr(9); mWordLengths(WordIndex);
            For SegmentIndex = 1 To mWordLengths(WordIndex)
                Print #2, Chr(9); mWordsBySegments(WordIndex, SegmentIndex);
            Next SegmentIndex
            Print #2,
        Next WordIndex
        Close
        End
        
    
End Sub

Sub FindLongestWord()
    
    Dim MyWord As String, MyCount As Long, DictFile As Long
    Let DictFile = FreeFile
    
    Open App.Path + "/FilesNeededForOperation/Dictionary.txt" For Input As #DictFile
    Do While Not EOF(DictFile)
        Line Input #DictFile, MyWord
        Let mNumberOfWordsInDictionary = mNumberOfWordsInDictionary + 1
        Let MyWord = Trim(MyWord)
        Let MyWord = s.Chomp(MyWord)
        Let MyCount = 0
        'Do
        '    If MyWord = "" Then
        '        If gLengthOfLongestWord < MyCount Then
        '            Let gLengthOfLongestWord = MyCount
        '        End If
        '        Exit Do
        '    Else
        '        Let MyWord = s.CustomResidue(MyWord, " ")
        '        Let MyCount = MyCount + 1
        '    End If
        'Loop
    Loop
    Close #DictFile

End Sub

Public Function CountNeighborsOneForm(MyWord As String) As String

    'Count and enumerate the lexical neighbors for one single word on the interface.
        Dim MyCount As Long
        
        If BLICK.ErrorCheck(MyWord, True) = "AllCharactersOK" Then
            Let MyCount = CountNeighbors(MyWord)
            Select Case MyCount
                Case 0
                    Let CountNeighborsOneForm = "The word  " + MyWord + " has no neighbors."
                Case 1
                    Let CountNeighborsOneForm = "The number of neighbors of " + MyWord + " is 1." + gDoubleLineBreak + "The neighbor is:" + gDoubleLineBreak + BLICK.mReportResultsString + "."
                Case Else
                    Let CountNeighborsOneForm = "The number of neighbors of " + MyWord + " is " + Trim(Str(CountNeighbors(MyWord))) + "." + _
                        gDoubleLineBreak + "The neighbors are:" + gDoubleLineBreak + BLICK.mReportResultsString + "."
            End Select
        End If

End Function


